#ifndef QT_WAYLAND_GREENISLAND_SCREENCASTER
#define QT_WAYLAND_GREENISLAND_SCREENCASTER

#include "wayland-greenisland-screencaster-client-protocol.h"
#include <QByteArray>
#include <QString>

QT_BEGIN_NAMESPACE

namespace QtWayland {
    class  greenisland_screencaster
    {
    public:
        greenisland_screencaster(struct ::wl_registry *registry, int id, int version);
        greenisland_screencaster(struct ::greenisland_screencaster *object);
        greenisland_screencaster();

        virtual ~greenisland_screencaster();

        void init(struct ::wl_registry *registry, int id, int version);
        void init(struct ::greenisland_screencaster *object);

        struct ::greenisland_screencaster *object() { return m_greenisland_screencaster; }
        const struct ::greenisland_screencaster *object() const { return m_greenisland_screencaster; }

        bool isInitialized() const;

        enum error {
            error_bad_output = 0 // The client passed an invalid output
        };

        struct ::greenisland_screencast *capture(struct ::wl_output *output);

    private:
        struct ::greenisland_screencaster *m_greenisland_screencaster;
    };

    class  greenisland_screencast
    {
    public:
        greenisland_screencast(struct ::wl_registry *registry, int id, int version);
        greenisland_screencast(struct ::greenisland_screencast *object);
        greenisland_screencast();

        virtual ~greenisland_screencast();

        void init(struct ::wl_registry *registry, int id, int version);
        void init(struct ::greenisland_screencast *object);

        struct ::greenisland_screencast *object() { return m_greenisland_screencast; }
        const struct ::greenisland_screencast *object() const { return m_greenisland_screencast; }

        bool isInitialized() const;

        enum error {
            error_bad_buffer = 0 // Buffer doesn't match requirements
        };

        enum transform {
            transform_normal = 1, // No particular transform has been applied
            transform_y_inverted = 2 // Y axis is inverted
        };

        void destroy();
        void record(struct ::wl_buffer *buffer);

    protected:
        virtual void screencast_setup(int32_t width, int32_t height, int32_t stride, int32_t format);
        virtual void screencast_frame(struct ::wl_buffer *buffer, uint32_t time, int32_t transform);
        virtual void screencast_failed(int32_t error, struct ::wl_buffer *buffer);
        virtual void screencast_cancelled(struct ::wl_buffer *buffer);

    private:
        void init_listener();
        static const struct greenisland_screencast_listener m_greenisland_screencast_listener;
        static void handle_setup(
            void *data,
            struct ::greenisland_screencast *object,
            int32_t width,
            int32_t height,
            int32_t stride,
            int32_t format);
        static void handle_frame(
            void *data,
            struct ::greenisland_screencast *object,
            struct ::wl_buffer *buffer,
            uint32_t time,
            int32_t transform);
        static void handle_failed(
            void *data,
            struct ::greenisland_screencast *object,
            int32_t error,
            struct ::wl_buffer *buffer);
        static void handle_cancelled(
            void *data,
            struct ::greenisland_screencast *object,
            struct ::wl_buffer *buffer);
        struct ::greenisland_screencast *m_greenisland_screencast;
    };
}

QT_END_NAMESPACE

#endif
