#include "qwayland-server-greenisland-screencaster.h"

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    greenisland_screencaster::greenisland_screencaster(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    greenisland_screencaster::greenisland_screencaster(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    greenisland_screencaster::greenisland_screencaster(struct ::wl_resource *resource)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(resource);
    }

    greenisland_screencaster::greenisland_screencaster()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    greenisland_screencaster::~greenisland_screencaster()
    {
    }

    void greenisland_screencaster::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    void greenisland_screencaster::init(struct ::wl_resource *resource)
    {
        m_resource = bind(resource);
    }

    greenisland_screencaster::Resource *greenisland_screencaster::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    greenisland_screencaster::Resource *greenisland_screencaster::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void greenisland_screencaster::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::greenisland_screencaster_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    const struct wl_interface *greenisland_screencaster::interface()
    {
        return &::greenisland_screencaster_interface;
    }

    greenisland_screencaster::Resource *greenisland_screencaster::screencaster_allocate()
    {
        return new Resource;
    }

    void greenisland_screencaster::screencaster_bind_resource(Resource *)
    {
    }

    void greenisland_screencaster::screencaster_destroy_resource(Resource *)
    {
    }

    void greenisland_screencaster::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        greenisland_screencaster *that = static_cast<greenisland_screencaster *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void greenisland_screencaster::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        greenisland_screencaster *that = resource->screencaster_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->screencaster_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    greenisland_screencaster::Resource *greenisland_screencaster::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Q_ASSERT_X(!wl_client_get_object(client, id), "QWaylandObject bind", QStringLiteral("binding to object %1 more than once").arg(id).toLocal8Bit().constData());
        struct ::wl_resource *handle = wl_resource_create(client, &::greenisland_screencaster_interface, version, id);
        return bind(handle);
    }

    greenisland_screencaster::Resource *greenisland_screencaster::bind(struct ::wl_resource *handle)
    {
        Resource *resource = screencaster_allocate();
        resource->screencaster_object = this;

        wl_resource_set_implementation(handle, &m_greenisland_screencaster_interface, resource, destroy_func);
        resource->handle = handle;
        m_resource = resource;
        screencaster_bind_resource(resource);
        return resource;
    }

    const struct ::greenisland_screencaster_interface greenisland_screencaster::m_greenisland_screencaster_interface = {
        greenisland_screencaster::handle_capture
    };

    void greenisland_screencaster::screencaster_capture(Resource *, uint32_t, struct ::wl_resource *)
    {
    }


    void greenisland_screencaster::handle_capture(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t id,
        struct ::wl_resource *output)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<greenisland_screencaster *>(r->screencaster_object)->screencaster_capture(
            r,
            id,
            output);
    }

    greenisland_screencast::greenisland_screencast(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    greenisland_screencast::greenisland_screencast(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    greenisland_screencast::greenisland_screencast(struct ::wl_resource *resource)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(resource);
    }

    greenisland_screencast::greenisland_screencast()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    greenisland_screencast::~greenisland_screencast()
    {
    }

    void greenisland_screencast::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    void greenisland_screencast::init(struct ::wl_resource *resource)
    {
        m_resource = bind(resource);
    }

    greenisland_screencast::Resource *greenisland_screencast::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    greenisland_screencast::Resource *greenisland_screencast::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void greenisland_screencast::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::greenisland_screencast_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    const struct wl_interface *greenisland_screencast::interface()
    {
        return &::greenisland_screencast_interface;
    }

    greenisland_screencast::Resource *greenisland_screencast::screencast_allocate()
    {
        return new Resource;
    }

    void greenisland_screencast::screencast_bind_resource(Resource *)
    {
    }

    void greenisland_screencast::screencast_destroy_resource(Resource *)
    {
    }

    void greenisland_screencast::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        greenisland_screencast *that = static_cast<greenisland_screencast *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void greenisland_screencast::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        greenisland_screencast *that = resource->screencast_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->screencast_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    greenisland_screencast::Resource *greenisland_screencast::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Q_ASSERT_X(!wl_client_get_object(client, id), "QWaylandObject bind", QStringLiteral("binding to object %1 more than once").arg(id).toLocal8Bit().constData());
        struct ::wl_resource *handle = wl_resource_create(client, &::greenisland_screencast_interface, version, id);
        return bind(handle);
    }

    greenisland_screencast::Resource *greenisland_screencast::bind(struct ::wl_resource *handle)
    {
        Resource *resource = screencast_allocate();
        resource->screencast_object = this;

        wl_resource_set_implementation(handle, &m_greenisland_screencast_interface, resource, destroy_func);
        resource->handle = handle;
        m_resource = resource;
        screencast_bind_resource(resource);
        return resource;
    }

    const struct ::greenisland_screencast_interface greenisland_screencast::m_greenisland_screencast_interface = {
        greenisland_screencast::handle_destroy,
        greenisland_screencast::handle_record
    };

    void greenisland_screencast::screencast_destroy(Resource *)
    {
    }

    void greenisland_screencast::screencast_record(Resource *, struct ::wl_resource *)
    {
    }


    void greenisland_screencast::handle_destroy(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<greenisland_screencast *>(r->screencast_object)->screencast_destroy(
            r);
    }

    void greenisland_screencast::handle_record(
        ::wl_client *client,
        struct wl_resource *resource,
        struct ::wl_resource *buffer)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<greenisland_screencast *>(r->screencast_object)->screencast_record(
            r,
            buffer);
    }

    void greenisland_screencast::send_setup(int32_t width, int32_t height, int32_t stride, int32_t format)
    {
        send_setup(
            m_resource->handle,
            width,
            height,
            stride,
            format);
    }

    void greenisland_screencast::send_setup(struct ::wl_resource *resource, int32_t width, int32_t height, int32_t stride, int32_t format)
    {
        greenisland_screencast_send_setup(
            resource,
            width,
            height,
            stride,
            format);
    }


    void greenisland_screencast::send_frame(struct ::wl_resource *buffer, uint32_t time, int32_t transform)
    {
        send_frame(
            m_resource->handle,
            buffer,
            time,
            transform);
    }

    void greenisland_screencast::send_frame(struct ::wl_resource *resource, struct ::wl_resource *buffer, uint32_t time, int32_t transform)
    {
        greenisland_screencast_send_frame(
            resource,
            buffer,
            time,
            transform);
    }


    void greenisland_screencast::send_failed(int32_t error, struct ::wl_resource *buffer)
    {
        send_failed(
            m_resource->handle,
            error,
            buffer);
    }

    void greenisland_screencast::send_failed(struct ::wl_resource *resource, int32_t error, struct ::wl_resource *buffer)
    {
        greenisland_screencast_send_failed(
            resource,
            error,
            buffer);
    }


    void greenisland_screencast::send_cancelled(struct ::wl_resource *buffer)
    {
        send_cancelled(
            m_resource->handle,
            buffer);
    }

    void greenisland_screencast::send_cancelled(struct ::wl_resource *resource, struct ::wl_resource *buffer)
    {
        greenisland_screencast_send_cancelled(
            resource,
            buffer);
    }

}

QT_END_NAMESPACE
