#ifndef QT_WAYLAND_SERVER_GREENISLAND_SCREENCASTER
#define QT_WAYLAND_SERVER_GREENISLAND_SCREENCASTER

#include "wayland-server.h"
#include "wayland-greenisland-screencaster-server-protocol.h"
#include <QByteArray>
#include <QMultiMap>
#include <QString>

#ifndef WAYLAND_VERSION_CHECK
#define WAYLAND_VERSION_CHECK(major, minor, micro) \
    ((WAYLAND_VERSION_MAJOR > (major)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR > (minor)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR == (minor) && WAYLAND_VERSION_MICRO >= (micro)))
#endif

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    class  greenisland_screencaster
    {
    public:
        greenisland_screencaster(struct ::wl_client *client, int id, int version);
        greenisland_screencaster(struct ::wl_display *display, int version);
        greenisland_screencaster(struct ::wl_resource *resource);
        greenisland_screencaster();

        virtual ~greenisland_screencaster();

        class Resource
        {
        public:
            Resource() : screencaster_object(0), handle(0) {}
            virtual ~Resource() {}

            greenisland_screencaster *screencaster_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        static const struct ::wl_interface *interface();
        static QByteArray interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        enum error {
            error_bad_output = 0 // The client passed an invalid output
        };

    protected:
        virtual Resource *screencaster_allocate();

        virtual void screencaster_bind_resource(Resource *resource);
        virtual void screencaster_destroy_resource(Resource *resource);

        virtual void screencaster_capture(Resource *resource, uint32_t id, struct ::wl_resource *output);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::greenisland_screencaster_interface m_greenisland_screencaster_interface;

        static void handle_capture(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t id,
            struct ::wl_resource *output);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };

    class  greenisland_screencast
    {
    public:
        greenisland_screencast(struct ::wl_client *client, int id, int version);
        greenisland_screencast(struct ::wl_display *display, int version);
        greenisland_screencast(struct ::wl_resource *resource);
        greenisland_screencast();

        virtual ~greenisland_screencast();

        class Resource
        {
        public:
            Resource() : screencast_object(0), handle(0) {}
            virtual ~Resource() {}

            greenisland_screencast *screencast_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        static const struct ::wl_interface *interface();
        static QByteArray interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        enum error {
            error_bad_buffer = 0 // Buffer doesn't match requirements
        };

        enum transform {
            transform_normal = 1, // No particular transform has been applied
            transform_y_inverted = 2 // Y axis is inverted
        };

        void send_setup(int32_t width, int32_t height, int32_t stride, int32_t format);
        void send_setup(struct ::wl_resource *resource, int32_t width, int32_t height, int32_t stride, int32_t format);
        void send_frame(struct ::wl_resource *buffer, uint32_t time, int32_t transform);
        void send_frame(struct ::wl_resource *resource, struct ::wl_resource *buffer, uint32_t time, int32_t transform);
        void send_failed(int32_t error, struct ::wl_resource *buffer);
        void send_failed(struct ::wl_resource *resource, int32_t error, struct ::wl_resource *buffer);
        void send_cancelled(struct ::wl_resource *buffer);
        void send_cancelled(struct ::wl_resource *resource, struct ::wl_resource *buffer);

    protected:
        virtual Resource *screencast_allocate();

        virtual void screencast_bind_resource(Resource *resource);
        virtual void screencast_destroy_resource(Resource *resource);

        virtual void screencast_destroy(Resource *resource);
        virtual void screencast_record(Resource *resource, struct ::wl_resource *buffer);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::greenisland_screencast_interface m_greenisland_screencast_interface;

        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_record(
            ::wl_client *client,
            struct wl_resource *resource,
            struct ::wl_resource *buffer);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };
}

QT_END_NAMESPACE

#endif
